[Leer en español](README_es.md)

# Using `run.sh`

The `run.sh` script automates the deployment process for a `k8s` environment using `minikube`. It includes building `docker` images, customizing `postgres` versions, enabling `SSL`, and configuring replicas for the API.

## Requirements

- [minikube](https://minikube.sigs.k8s.io/docs/start/)
- [docker](https://docs.docker.com/get-docker/)
- [helm](https://helm.sh/docs/intro/install/)

## Usage

```bash
./run.sh -i  # Interactive mode
./run.sh -p <version> -s <Y|N> -r <number>  # Non-interactive mode
./run.sh -p <version> -s <Y|N> -r <number> -f  # Non-interactive mode with Fluentd enabled
```

## Command Line Options

- **-i:** Interactive mode
- **-p <version>:** Specify the `postgres` version
- **-s <Y|N>:** Enable or disable `SSL` (Y for yes, N for no)
- **-r <number>:** Specify the number of replicas for the API
- **-f:** Enable `fluentd` for log aggregation
- **-u:** Uninstall the chart
- **-d:** Add basic data to the DB

## Examples

### Changing postgres version

If you want to modify the database version, you can simply do:

```bash
./run.sh -p 13.1
```

This will update everything necessary for this change.

### Specifying number of replicas for the API

```bash
./run.sh -r 6
```

## Script Functionality

1. **Minikube Status Check:**

   The script checks if `minikube` is already running and starts it if it's not.

2. **Postgres Version and Build:**

   It prompts the user for the postgres version, builds `docker` images, and updates the version in the `helm` values file.

3. **TLS Configuration:**

   It asks the user if they want to enable `TLS` and updates the `helm` values file accordingly.

4. **Configuring Replicas for the API:**

   It prompts the user for the number of replicas for the API and updates the `helm` values file.

5. **Ingress Configuration:**

   It enables the ingress extension if `minikube` is running.

6. **Fluentd Configuration (Optional):**
  
   The user is asked whether to enable `fluentd` for log aggregation. This can be toggled directly in the `values.yaml` file. It's worth noting that, to maintain simplicity and avoid "bloat" in `values.yaml`, an additional file named `fluentd.yaml` is provided. This file should be passed as a parameter `-f` to `helm` to configure `fluentd` with necessary parameters. It's optional, as `Fluentd` can function without these configurations, but it's recommended due to performing some container startup conditions to wait for the `fluentd` service and to collect only logs from `exam-*` containers (not all logs from `k8s`).

   The following instruction is used when installing or upgrading `helm`:

   ```bash
   helm install exam ./helm -n exam -f helm/values.yaml -f helm/fluentd.yaml
   ```

   To view logs from `exam-*` containers, you can use:

   ```bash
   kubectl logs -n exam exam-fluentd-0
   ```

   To configure and send logs to a log backend like `elasticsearch`, you can edit `fluentd.yaml` as per your requirements.

7. **Secrets Configuration (Optional):**

   To avoid keeping secrets in the SVC (if one is used), they can be passed to `helm` via a `secrets.yaml` file with the `-f` option. If `./helm/secrets.yaml` exists (in the `./helm` folder, i.e., in the same location as `values.yaml`), these values will be used; otherwise, those defined in `values.yaml` will be used. These values will override existing ones.

   It's worth noting that upon creating `./helm/secrets.yaml`, the `run.sh` script will automatically detect and use it when running `helm`. Therefore, no additional actions are required, as the script will handle the detection and passing of this file to `helm`.
